# Apollopedia

This package is used to highlight key terms in blocks of text within our education websites, and explain what those terms mean in a popover on click.

The terms themselves are maintained as markdown files within the `terms` directory. The file name is the term that should be highlighted and the content of the markdown file is used as the explanation. This makes it easy to apply formatting and write code blocks within the explanation. It also provides a boost for maintainers, improving readability when reviewing changes or approving new terms.

## Installation

```sh
yarn add @apollo/pedia
```

## Usage

In its simplest form, the `HighlightKeyTerms` component exported by the `@apollo/pedia` package takes some text as its children and automatically highlights key terms within it.

```tsx
import { HighlightKeyTerms } from "@apollo/pedia";

export const MyComponent = () => {
  return (
    <div>
      <h1>welcome to my website</h1>
      <p>
        <HighlightKeyTerms>
          this is a some text that contains a key term, like subgraph
        </HighlightKeyTerms>
      </p>
    </div>
  );
};
```

### With MDX

But if you're using MDX, you can use the component substitution API to automatically highlight key terms that appear anywhere in the page.

```tsx
import { HighlightKeyTerms } from "@apollo/pedia";

const components = {
  // wrap the children of paragraphs in our highlighter
  p: ({ children }: JSX.IntrinsicElements["p"]) => {
    return (
      <p>
        <HighlightKeyTerms>{children}</HighlightKeyTerms>
      </p>
    );
  },
  // you can wrap other elements too, like list items
  li: ({ children }: JSX.IntrinsicElements["li"]) => {
    return (
      <li>
        <HighlightKeyTerms>{children}</HighlightKeyTerms>
      </li>
    );
  },
};

export const MdxTemplate = () => {
  return <MdxProvider components={components}></MdxProvider>;
};
```
